
! (c) sparslab module name=sainv_spd_left_main
!
! purpose:
!  a sample driver for symmetric left-looking SAINV
!
!  this driver does the following:
!
!     -- reads a simple matrix A
!     -- symmetrizes it for input to SAINV
!     -- computes the decompositon M \approx inv(A)
!     -- computes M * (1, ..., 1)^T
!     -- outputs the result as 'solution x'
!
      use ainv90
!
      implicit none
      integer n,m,i,iunit,ierr
      double precision zero
      parameter(zero=0.0d0)
      integer msglvl,msgunit,size_p
      integer, allocatable :: ip(:),jp(:)
      double precision, allocatable :: ap(:)
      integer droptyp
      integer imodif,diag_one,info
      integer fill,mformat
      double precision drflai,diagtol
      integer, allocatable, dimension(:) :: ia,ja,wn01,wn02
      double precision, allocatable, dimension(:) :: aa,x,y,wr01
      double precision, allocatable :: diag(:)
      double precision maxdiag,mindiag,maxznorm,minznorm
!
! start of sainv_spd_left_main
!
!
!  -- read the matrix (n,ia,ja,aa) in
!
!
      open(unit=iunit,file='bcsstk04.rsa')
      call imhb2(iunit,m,n,mformat,ia,ja,aa,info)
      if(mformat.eq.111) then ! symmetrize
        allocate(wn01(n+1),wn02(n+1),stat=ierr)
        call symtr7(n,ia,ja,aa,wn01,wn02)
        mformat=11
        deallocate(wn01,wn02)
      end if
!
!  -- get sample right-hand side
!
      allocate(x(n),y(n))
      do i=1,n
        x(i)=dble(i)
      end do
      call mvucsr1(n,ia,ja,aa,x,y)
!
!     %%%%%%%%%%%%%%%%%
!  -- to be set by user
!     %%%%%%%%%%%%%%%%%
!
!
!  -- set the drop tolerance
!  -- for the sainv processs
!
      drflai=0.0d0
!
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!  -- end of the part to be set by user
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!
!
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!  -- start of defaults -- need not to be changed
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!
!  -- sizes: an initial first guess
!  --        will be automatically adapted inside the code
!
      size_p=1000000
!
!  -- other parameters without the need to be changed
!
      diagtol=1.1d-16
      droptyp=0
      diag_one=1
!
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!  -- end of defaults -- need not to be changed
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!
!
!  -- standard level of output
!
      msglvl=2
!
!  -- open unit for output
!
      msgunit=2
      open(unit=msgunit,file='output.out')
!
!  -- call sainv
!
      call ainvsl3(msglvl,msgunit,n,
     *  ia,ja,aa,ip,jp,ap,
     *  size_p,diag,diagtol,
     *  drflai,imodif,diag_one,droptyp,fill,
     *  maxznorm,minznorm,maxdiag,mindiag,info)
      deallocate(ia,ja,aa)
!
!  -- output preconditioner info
!
      write(2,*)' preconditioner successfully computed'
      write(2,*)' drop tolerance',drflai
      write(2,*)' preconditioner size',ip(n+1)-1
      write(2,*)' pointers'
      write(2,*) ip(1:n+1)
      write(2,*)' column indices'
      write(2,*) jp(1:ip(n+1)-1)
      write(2,*)' nonzero entries'
      write(2,*) ap(1:ip(n+1)-1)
!
!  -- solve the system
!
      allocate(wr01(n))
      call mvucsr1(n,ip,jp,ap,y,wr01)
      call tvucsr1(n,n,ip,jp,ap,wr01,x)
!
!  -- print solution x
!
      write(2,*)' solution x'
      write(2,*) x(1:n)
!
      deallocate(x,y,wr01)
!
      close(2)
      stop ' success'
!
! end of sainv_spd_left_main
!
      end

